@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+import hashlib
 import os
 import re
 import subprocess
@@ -37,12 +40,10 @@ def print_progress_bar(
     filled_length = int(length * iteration // total)
     bar = fill * filled_length + '-' * (length - filled_length)
     print(f'\r{prefix} |{bar}| {percent}% {suffix}', end=print_end)
-    if iteration == total:
-        print()
 
 
-def reset_term_color():
-    print(Style.RESET_ALL, end="")
+def reset_term_color(new_line: bool = False):
+    print(Style.RESET_ALL, end="\n" if new_line else "")
 
 
 def port_abbreviation(port: str):
@@ -54,3 +55,28 @@ def port_abbreviation(port: str):
         elif port[0] == "u":
             port = "/dev/ttyUSB" + port[1:]
     return port
+
+
+def recursive_list_dir(path: str) -> tuple[dict[str, str], dict[str, str]]:
+    out_dirs = {}
+    out_files = {}
+    for abs_dir, dirs, files in os.walk(replace_backslashes(path)):
+        abs_dir = replace_backslashes(abs_dir)
+        rel_dir = abs_dir.removeprefix(path)
+        for dir_name in dirs:
+            out_dirs[f"{rel_dir}/{dir_name}"] = f"{abs_dir}/{dir_name}"
+        for file_name in files:
+            out_files[f"{rel_dir}/{file_name}"] = f"{abs_dir}/{file_name}"
+    return out_dirs, out_files
+
+
+def get_file_sha1(path: str) -> bytes:
+    with open(path, "rb") as file:
+        return hashlib.sha1(file.read()).digest()
+
+
+def get_temp_dirname_prefix(full_port: str):
+    return "mpbridge-" + remove_prefix(
+        full_port, "/dev/").replace(
+        "tty", "").replace(
+        "/", "-") + "-"
